import { relative, resolve } from 'node:path';
import { rm, run, resolveBin } from '@genesislcap/build-kit';
import type { BuildContext } from '@genesislcap/build-kit';

export default async (ctx: BuildContext) => {
  const {
    dirs: { cwd },
    cli: { isTest, options },
    pkg,
  } = ctx;
  const { coverage, watch, debug, e2e, interactive, browser, match, rawMatch } = options;

  if (!isTest) {
    throw new Error(`Unrecognized command: ${JSON.stringify(options)}`);
  }

  if (coverage) {
    await rm(['coverage']);
  }

  // E2E tests
  if (e2e) {
    const uiParam = interactive ? '--ui' : '';
    const debugParam = debug ? '--debug' : '';

    run(cwd, `npx playwright test --config=./playwright.config.ts ${uiParam} ${debugParam}`, {
      prefixCommandWithNode: false,
      env: {
        ...process.env,
        PORT: pkg.config?.PORT,
      },
    });
    return;
  } else {
    // unit tests
    if (browser) {
      // browser target - executing using playwright-test
      const playwrightTest = await resolveBin('playwright-test');
      const debugParam = debug ? '--debug' : '';
      const watchParam = watch ? '--watch' : '';
      const covParam = coverage ? '--cov' : '';
      let matchParam = './**/*.test.ts';
      if (match) {
        matchParam = rawMatch ? match : `./**/${match}`;
      }
      const command = `${playwrightTest} "${matchParam}"  --runner uvu ${debugParam} ${watchParam} ${covParam}`;
      run(cwd, command);

      if (coverage) {
        const nyc = await resolveBin('nyc');
        const nycCommand = `${nyc} report --reporter=text --reporter=lcov`;
        run(cwd, nycCommand);
      }
    } else {
      // node target - executing using uvu/jsdom
      const nodeArgs = debug ? '--inspect-brk' : '';
      const matchParam = match ? match : 'test.ts';
      // tsm expects a relative path to config file incl. extension
      const tsmJS = resolve(__dirname, './config/tsm.js');
      const tsmJSRelative = relative(cwd, tsmJS);
      const jsdomSetupPath = resolve(__dirname, './config/jsdom.setup.ts');
      const uvu = await resolveBin('uvu');
      const ignore = ['dist', 'e2e', 'docs', 'temp', 'coverage']; // .git, node_modules are ignored by default
      const unitTestCommand = `${uvu} -r tsm --tsmconfig ${tsmJSRelative} -r @esbuild-kit/cjs-loader -r ${jsdomSetupPath} . "${matchParam}" ${ignore.map(
        (i) => ' -i ' + i,
      )}`;

      if (coverage) {
        const c8 = await resolveBin('c8');
        const command = `${c8} --include=src --all  --reporter=text --reporter=lcov node ${unitTestCommand}`;
        run(cwd, command);
      } else if (watch) {
        const watchlist = await resolveBin('watchlist');
        run(cwd, `${watchlist} src test -- ${unitTestCommand}`);
      } else {
        run(cwd, unitTestCommand, { nodeArgs });
      }
    }
  }
};
